#include "nibobee.h"
#include "servo.h"
#include <avr/interrupt.h>

// Konfigurations-Einstellungen für die Servos

// servox_pin(level) Makro, um die Steuerleitung des Servos auf low oder high zu setzen.
// servox_output     Makro, um die Steuerleitung des Servos in den Ausgabemodus zu setzen (Initialisierung)
// servox_center     PWM Wert für mittlere Position
// servox_factor     PWM Wert pro Grad Drehung, Fließkommazahlen bitte als Bruch Schreiben

// Servo 0, Typ Modelcraft ES-05
#define servo0_pin(level) writeBit(PORTC,2,level)
#define servo0_output writeBit(DDRC,2,1)
#define servo0_center 76
#define servo0_factor 6/10

// Unbenutzte Kanäle müssen auskommentiert werden

// Zur Dokumentation der Anwendungs-Schnittstelle siehe servo.h

//#define servo1_pin(level) writeBit(PORTx,pin,level)
//#define servo1_output writeBit(DDRx,pin,level)
//#define servo1_center 88
//#define servo1_factor 6/10

//#define servo2_pin(level) writeBit(PORTx,pin,level)
//#define servo2_output writeBit(DDRx,pin,level)
//#define servo2_center 88
//#define servo2_factor 6/10

//#define servo3_pin(level) writeBit(PORTx,pin,level)
//#define servo3_output writeBit(DDRx,pin,level)
//#define servo3_center 88
//#define servo3_factor 6/10

//#define servo4_pin(level) writeBit(PORTx,pin,level)
//#define servo4_output writeBit(DDRx,pin,level)
//#define servo4_center 88
//#define servo4_factor 6/10



// PWM Werte für die Servos. Der spezielle Wert 0 schaltet den Servo aus.
volatile uint8_t servo[5];


// Schaltet einen der bis zu 5 Servos in den Standby Modus.
void servo_standby(uint8_t kanal) {
    servo[kanal]=0;
}


// Bestimmt die Zielposition für einen der bis zu 5 Servos.
// Servos unterstützen typischwerweise den Bereich -90 bis +90 Grad.
void set_servo(uint8_t kanal, int8_t winkel) {
    switch (kanal) {
        #ifdef servo0_center
            case 0: servo[0]=(int16_t)winkel*servo0_factor+servo0_center; break;
        #endif
        #ifdef servo1_center
            case 1: servo[1]=(int16_t)winkel*servo1_factor+servo1_center; break;
        #endif
        #ifdef servo2_center
            case 2: servo[2]=(int16_t)winkel*servo2_factor+servo2_center; break;
        #endif
        #ifdef servo3_center
            case 3: servo[3]=(int16_t)winkel*servo3_factor+servo3_center; break;
        #endif
        #ifdef servo4_center
            case 4: servo[4]=(int16_t)winkel*servo4_factor+servo4_center; break;
        #endif
    }
}


// Timer Compare Interrupt schaltet den Steuerimpuls aus.
#ifdef TIMER2_COMPA_vect
    ISR(TIMER2_COMPA_vect) { // ATmega644
#else
    ISR(TIMER2_COMP_vect) { // ATmega16
#endif
    #ifdef servo0_pin
        servo0_pin(0);
    #endif
    #ifdef servo1_pin
        servo1_pin(0);
    #endif
    #ifdef servo2_pin
        servo2_pin(0);
    #endif
    #ifdef servo3_pin
        servo3_pin(0);
    #endif
    #ifdef servo4_pin
        servo4_pin(0);
    #endif
}


// Timer Overflow Interrupt beginnt einen Steuerimpuls, bei jedem
// Durchlauf wird der jeweils nächste Servo angesteuert.
ISR(TIMER2_OVF_vect) {
    static uint8_t channel=0;
    uint8_t value=servo[channel];
    if (value>0) {
        switch (channel) {
            #ifdef servo0_pin
                case 0: servo0_pin(1); break;
            #endif
            #ifdef servo1_pin
                case 1: servo1_pin(1); break;
            #endif
            #ifdef servo2_pin
                case 2: servo2_pin(1); break;
            #endif
            #ifdef servo3_pin
                case 3: servo3_pin(1); break;
            #endif
            #ifdef servo4_pin
                case 4: servo4_pin(1); break;
            #endif
        }
    }
    #ifdef OCR2A
        OCR2A=value; // Atmega644
    #else
        OCR2=value; // Atmega16
    #endif
    if (++channel==5) { // Nicht ändern, auch nicht, wenn Du weniger als 5 Kanäle benutzt!
        channel=0;
    }
}


// Installiert die servo_init() Prozedur in den Start-Bereich 5,
// so dass sie automatisch vor main() ausgeführt wird.
void servo_init() __attribute__ ((naked))  __attribute__ ((section (".init5")));

// Initialisiert die Hardware
void servo_init() {
    #ifdef servo0_output
        servo0_output;
    #endif
    #ifdef servo1_output
        servo1_output;
    #endif
    #ifdef servo2_output
        servo2_output;
    #endif
    #ifdef servo3_output
        servo3_output;
    #endif
    #ifdef servo4_output
        servo4_output;
    #endif
    
    // Timer 2, normal mode, prescaler 256, interrupt on overflow and compare match A.
    #ifdef TCCR2A
        // Für neue AVR Modelle
        TCCR2A = 0;
        TCCR2B = (1<<CS22) | (1<<CS21); 
        TIMSK2 = (1<<TOIE2) | (1<<OCIE2A);
    #else
        // Für den Atmega16
        TCCR2 = (1<<CS22) | (1<<CS21);
        TIMSK |= (1<<TOIE2) | (1<<OCIE2);
    #endif
}